==> wmc_plugin-1.0.0.tar.gz: ios/Classes/ApiWmcModule.swift <==
//
//  ApiWmcModule.swift
//  Runner
//
//  Created by Jorge Tlahuechtl Juarez on 30/04/20.
//  Copyright © 2020 The Chromium Authors. All rights reserved.
//

import Foundation
import Alamofire

class ApiWmcModule{
    
    static let shared = ApiWmcModule()
    
    let UrlWMC = "https://api.worldmobileconnection.com/v2/"
    
    var requestManager = Alamofire.SessionManager.default
    
    
    /**
        Método que inicializa WMC
     */
    func initialize(apiKey:String, completion: @escaping (Result<String>) -> Void){
        if !apiKey.isEmpty{
            
            Utils.setApiKey(apikey: apiKey)
            
            getToken(ApiKey: apiKey) { (result) in
                switch (result){
                case .Success(_):
                        completion(.Success("Inicializacón correcta"))
                    case.Error(let error):
                        completion(.Error(error))
                }
            }
        }
    }
    
    /**
       Método que obtiene un token valido para usarse en la api de WMC
    */
    func getToken(ApiKey:String,completion: @escaping (Result<Token>) -> Void){
        Alamofire.request(self.UrlWMC+"gettoken", method: .post, parameters: ["apikey":ApiKey], encoding: URLEncoding.default, headers: nil).responseJSON { (response:DataResponse<Any>) in
            
            switch(response.result) {
            case .success(_):
                
                //let responseString: String? = String(data: response.data!, encoding: .utf8)
                let token = try! JSONDecoder().decode(Token.self, from: response.data!)
                
                
                print(token)
                completion(.Success(token))
                
                
            case .failure(_):
                print("error gettoken")
                print(response.result.error as Any)
                completion(.Error((response.response?.statusCode) ?? 999))
            }
        }
    }
    
    /**
    * Obtiene una session valida otorgada por WMC
    */
    func getUser(token:String,completion: @escaping (Result<Session>) -> Void){
        
        let mcc = Utils.getMcc()
        let mnc = Utils.getMnc()
        let uuid = Utils.getUUID()
        let bundleID = Bundle.main.bundleIdentifier!
        let nsObject: AnyObject? = Bundle.main.infoDictionary!["CFBundleShortVersionString"] as AnyObject?
        let version = nsObject as! String
        let network = Utils.getNetwork()
        let device_os = "iOS \(UIDevice.current.systemVersion)"
        let locale = NSLocale.current.languageCode
        let transport = Utils.getTypeConnection()
        
        let parameters: [String: String] = [
            "token":token ,
            "mcc":mcc ,
            "mnc":mnc ,
            "uniqid":uuid ,
            "transport":transport ,
            "device_os":device_os ,
            "device_name":UIDevice.current.modelName ,
            "appid":bundleID ,
            "app_Version":version ,
            "language": locale ?? "es" ,
            "network_type":network ]
        
        //print("tokenuser = \(parameters)")
        
        
        Alamofire.request(self.UrlWMC+"getuser", method: .post,
                          parameters: parameters,
            encoding: URLEncoding.default, headers: nil).responseData {
                            (response:DataResponse<Data>) in
                            
                            switch(response.result) {
                            case .success(_):
                                //let responseString: String? = String(data: response.data!, encoding: .utf8)
                                
                                var session = try! JSONDecoder().decode(Session.self, from: response.data!)
                                session.created = Utils.currentTimeInMiliseconds()
                                session.transport = transport
                                Utils.setSession(session: session)
                                completion(.Success(session))
                                break
                                
                            case .failure(_):
                                print("Error api")
                                print(response.result.error as Any)
                                completion(.Error((response.response?.statusCode) ?? 999))
                                break
                            }
        }
        
    }
    
    
    /**
    * Método que ejecuta un petición
    * @param uri Url a realizar el request
    * @param method Metodo GET o POST
    * @param sesion objecto que contiene la información de sesion para conexión con el proxy
    * @param options
    * @param parameters
    * @param listener
    */
     
    func request(uri:String,method:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        let transport = Utils.getTypeConnection()
        //Obtiene y verifica session valida
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self.sendRequest(session,uri: uri, method: method, options: options, data: data, completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self.sendRequest(session!, uri: uri, method: method, options: options, data: data, completion: completion)
        }
    }
    
    
    private func sendRequest(_ session:Session, uri:String,method:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        
        putProxyRequest(session: session)
        
        if method == "GET"{
            self.get(uri: uri, options: options, data: data, completion: completion)
        }else if method == "POST"{
            self.post(uri: uri, options: options, data: data, completion: completion)
        }else if method == "PUT"{
            self.put(uri: uri, options: options, data: data, completion: completion)
        }else if method == "DELETE"{
            self.delete(uri: uri, options: options, data: data, completion: completion)
        }
    }
    
    func getResource(uri:String,completion: @escaping (Result<Data>) -> Void){
        let transport = Utils.getTypeConnection()
        
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self.getData(session: session, uri: uri,completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self.getData(session: session!, uri: uri,completion: completion)
        }
    }
    
    
    private func getData(session:Session, uri:String,completion: @escaping (Result<Data>) -> Void){
        putProxyRequest(session: session)
        requestManager.request(uri).responseData { (resposnse:DataResponse<Data>) in
            switch(resposnse.result){
                case .success(let data):
                    completion(.Success(data))
                case .failure(_):
                    completion(.Error(resposnse.response?.statusCode ?? 999))
            }
        }
    }
    
    
    private func get(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .get, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }
    
    private func post(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .post, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }


    private func put(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .put, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }

    private func delete(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .delete, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }
    
    /**
     Método que configura el proxy de WMC
     */
    func putProxyRequest(session:Session){
        let configuration = URLSessionConfiguration.default
        
        configuration.timeoutIntervalForRequest = 10 // seconds
        configuration.timeoutIntervalForResource = 10
        
        configuration.httpAdditionalHeaders = SessionManager.defaultHTTPHeaders
        
        let loginData = String(format: "%@:%@", session.username, session.password).data(using: String.Encoding.utf8)!
        let base64LoginData = loginData.base64EncodedString()
        let headers = ["Proxy-Authorization" : "Basic \(base64LoginData)",
            "Content-Type": "application/json",
            "sessionid": session.sessionid ]
        
        configuration.httpAdditionalHeaders = headers as [AnyHashable : Any]
        
        let urlProxyFull = session.gw.deletingPrefix("https://").deletingPrefix("http://")
        
        let urlProxy = urlProxyFull.split(separator: ":")[0]
        let portProxy = urlProxyFull.split(separator: ":")[1]
        
        //print("token = \(token)")
        
        
        var proxyConfiguration = [String: AnyObject]()
        proxyConfiguration.updateValue(1 as AnyObject , forKey: "HTTPEnable")
        proxyConfiguration.updateValue(String(urlProxy) as String as AnyObject , forKey: "HTTPProxy")
        proxyConfiguration.updateValue(Int(portProxy) as AnyObject , forKey: "HTTPPort")
        proxyConfiguration.updateValue(1 as AnyObject , forKey: "HTTPSEnable")
        proxyConfiguration.updateValue(String(urlProxy) as String as AnyObject , forKey: "HTTPSProxy")
        proxyConfiguration.updateValue(Int(portProxy) as AnyObject , forKey: "HTTPSPort")
        
        configuration.connectionProxyDictionary = proxyConfiguration
        
        requestManager = Alamofire.SessionManager(configuration: configuration)
    }
    
    
}

enum Result<T> {
    case Success(T)
    case Error(Int)
}

extension String {
    func deletingPrefix(_ prefix: String) -> String {
        guard self.hasPrefix(prefix) else { return self }
        return String(self.dropFirst(prefix.count))
    }
}

==> wmc_plugin-1.0.2.tar.gz: ios/Classes/ApiWmcModule.swift <==
//
//  ApiWmcModule.swift
//  Runner
//
//  Created by Jorge Tlahuechtl Juarez on 30/04/20.
//  Copyright © 2020 The Chromium Authors. All rights reserved.
//

import Foundation
import Alamofire

class ApiWmcModule{
    
    static let shared = ApiWmcModule()
    
    let UrlWMC = "https://api.worldmobileconnection.com/v2/"
    
    var requestManager = Alamofire.SessionManager.default
    
    
    /**
        Método que inicializa WMC
     */
    func initialize(apiKey:String, completion: @escaping (Result<String>) -> Void){
        if !apiKey.isEmpty{
            
            Utils.setApiKey(apikey: apiKey)
            
            getToken(ApiKey: apiKey) { (result) in
                switch (result){
                case .Success(_):
                        completion(.Success("Inicializacón correcta"))
                    case.Error(let error):
                        completion(.Error(error))
                }
            }
        }
    }
    
    /**
       Método que obtiene un token valido para usarse en la api de WMC
    */
    func getToken(ApiKey:String,completion: @escaping (Result<Token>) -> Void){
        Alamofire.request(self.UrlWMC+"gettoken", method: .post, parameters: ["apikey":ApiKey], encoding: URLEncoding.default, headers: nil).responseJSON { (response:DataResponse<Any>) in
            
            switch(response.result) {
            case .success(_):
                
                //let responseString: String? = String(data: response.data!, encoding: .utf8)
                let token = try! JSONDecoder().decode(Token.self, from: response.data!)
                
                
                print(token)
                completion(.Success(token))
                
                
            case .failure(_):
                print("error gettoken")
                print(response.result.error as Any)
                completion(.Error((response.response?.statusCode) ?? 999))
            }
        }
    }
    
    /**
    * Obtiene una session valida otorgada por WMC
    */
    func getUser(token:String,completion: @escaping (Result<Session>) -> Void){
        
        let mcc = Utils.getMcc()
        let mnc = Utils.getMnc()
        let uuid = Utils.getUUID()
        let bundleID = Bundle.main.bundleIdentifier!
        let nsObject: AnyObject? = Bundle.main.infoDictionary!["CFBundleShortVersionString"] as AnyObject?
        let version = nsObject as! String
        let network = Utils.getNetwork()
        let device_os = "iOS \(UIDevice.current.systemVersion)"
        let locale = NSLocale.current.languageCode
        let transport = Utils.getTypeConnection()
        
        let parameters: [String: String] = [
            "token":token ,
            "mcc":mcc ,
            "mnc":mnc ,
            "uniqid":uuid ,
            "transport":transport ,
            "device_os":device_os ,
            "device_name":UIDevice.current.modelName ,
            "appid":bundleID ,
            "app_Version":version ,
            "language": locale ?? "es" ,
            "network_type":network ]
        
        //print("tokenuser = \(parameters)")
        
        
        Alamofire.request(self.UrlWMC+"getuser", method: .post,
                          parameters: parameters,
            encoding: URLEncoding.default, headers: nil).responseData {
                            (response:DataResponse<Data>) in
                            
                            switch(response.result) {
                            case .success(_):
                                //let responseString: String? = String(data: response.data!, encoding: .utf8)
                                
                                var session = try! JSONDecoder().decode(Session.self, from: response.data!)
                                session.created = Utils.currentTimeInMiliseconds()
                                session.transport = transport
                                Utils.setSession(session: session)
                                completion(.Success(session))
                                break
                                
                            case .failure(_):
                                print("Error api")
                                print(response.result.error as Any)
                                completion(.Error((response.response?.statusCode) ?? 999))
                                break
                            }
        }
        
    }
    
    
    /**
    * Método que ejecuta un petición
    * @param uri Url a realizar el request
    * @param method Metodo GET o POST
    * @param sesion objecto que contiene la información de sesion para conexión con el proxy
    * @param options
    * @param parameters
    * @param listener
    */
     
    func request(uri:String,method:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        let transport = Utils.getTypeConnection()
        //Obtiene y verifica session valida
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self.sendRequest(session,uri: uri, method: method, options: options, data: data, completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self.sendRequest(session!, uri: uri, method: method, options: options, data: data, completion: completion)
        }
    }
    
    
    private func sendRequest(_ session:Session, uri:String,method:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        
        putProxyRequest(session: session)
        
        if method == "GET"{
            self.get(uri: uri, options: options, data: data, completion: completion)
        } else if method == "DELETE"{
            self.delete(uri: uri, options: options, data: data, completion: completion)
        }
    }
    
    func getResource(uri:String,completion: @escaping (Result<Data>) -> Void){
        let transport = Utils.getTypeConnection()
        
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self.getData(session: session, uri: uri,completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self.getData(session: session!, uri: uri,completion: completion)
        }
    }
    
    
    private func getData(session:Session, uri:String,completion: @escaping (Result<Data>) -> Void){
        putProxyRequest(session: session)
        requestManager.request(uri).responseData { (resposnse:DataResponse<Data>) in
            switch(resposnse.result){
                case .success(let data):
                    completion(.Success(data))
                case .failure(_):
                    completion(.Error(resposnse.response?.statusCode ?? 999))
            }
        }
    }
    
    
    private func get(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .get, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }
    
    func post(uri:String,options:Dictionary<String,String>,data:String,completion: @escaping (Result<String>) -> Void){
        let transport = Utils.getTypeConnection()
        //Obtiene y verifica session valida
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self._post(session,uri: uri, options: options, data: data, completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self._post(session!, uri: uri, options: options, data: data, completion: completion)
        }
        
    }
    
    private func _post(_ session:Session,uri:String,options:Dictionary<String,String>,data:String,completion: @escaping (Result<String>) -> Void){
        putProxyRequest(session: session)
        
        
        requestManager.request(uri, method: .post, parameters: [:], encoding: JSONStringArrayEncoding.init(string: data), headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }


    func put(uri:String,options:Dictionary<String,String>,data:String,completion: @escaping (Result<String>) -> Void){
        let transport = Utils.getTypeConnection()
        //Obtiene y verifica session valida
        let session = Utils.getSession(transport: transport)
        
        if session == nil {
            let apiKey = Utils.getApiKey()
            getToken(ApiKey: apiKey) { (reponse) in
                switch (reponse){
                    case .Success(let obj):
                        let token = obj.Token
                        self.getUser(token: token) { (r) in
                            switch(r){
                                case .Success(let session):
                                    self._put(session,uri: uri, options: options, data: data, completion: completion)
                                case .Error(let error):
                                    completion(.Error(error))
                            }
                    }
                    case .Error(let error):
                        completion(.Error(error))
                }
            }
        }else{
            self._put(session!, uri: uri, options: options, data: data, completion: completion)
        }
    }
    
    
    private func _put(_ session:Session,uri:String,options:Dictionary<String,String>,data:String,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .put, parameters: [:], encoding: JSONStringArrayEncoding.init(string: data), headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }

    private func delete(uri:String,options:Dictionary<String,String>,data:Dictionary<String,String>,completion: @escaping (Result<String>) -> Void){
        requestManager.request(uri, method: .delete, parameters: data, encoding: URLEncoding.default, headers: options).responseJSON { (response:DataResponse<Any>) in
        
                switch(response.result) {
                case .success(_):
                    let responseString: String? = String(data: response.data!, encoding: .utf8)
                    completion(.Success(responseString ?? ""))
                    break
                    
                case .failure(_):
                    print("Error api")
                    print(response.result.error as Any)
                    completion(.Error((response.response?.statusCode) ?? 999))
                    break
                }
        }
    }
    
    /**
     Método que configura el proxy de WMC
     */
    func putProxyRequest(session:Session){
        let configuration = URLSessionConfiguration.default
        
        configuration.timeoutIntervalForRequest = 10 // seconds
        configuration.timeoutIntervalForResource = 10
        
        configuration.httpAdditionalHeaders = SessionManager.defaultHTTPHeaders
        
        let loginData = String(format: "%@:%@", session.username, session.password).data(using: String.Encoding.utf8)!
        let base64LoginData = loginData.base64EncodedString()
        let headers = ["Proxy-Authorization" : "Basic \(base64LoginData)",
            "Content-Type": "application/json",
            "sessionid": session.sessionid ]
        
        configuration.httpAdditionalHeaders = headers as [AnyHashable : Any]
        
        let urlProxyFull = session.gw.deletingPrefix("https://").deletingPrefix("http://")
        
        let urlProxy = urlProxyFull.split(separator: ":")[0]
        let portProxy = urlProxyFull.split(separator: ":")[1]
        
        //print("token = \(token)")
        
        
        var proxyConfiguration = [String: AnyObject]()
        proxyConfiguration.updateValue(1 as AnyObject , forKey: "HTTPEnable")
        proxyConfiguration.updateValue(String(urlProxy) as String as AnyObject , forKey: "HTTPProxy")
        proxyConfiguration.updateValue(Int(portProxy) as AnyObject , forKey: "HTTPPort")
        proxyConfiguration.updateValue(1 as AnyObject , forKey: "HTTPSEnable")
        proxyConfiguration.updateValue(String(urlProxy) as String as AnyObject , forKey: "HTTPSProxy")
        proxyConfiguration.updateValue(Int(portProxy) as AnyObject , forKey: "HTTPSPort")
        
        configuration.connectionProxyDictionary = proxyConfiguration
        
        requestManager = Alamofire.SessionManager(configuration: configuration)
    }
    
    
}

enum Result<T> {
    case Success(T)
    case Error(Int)
}

struct JSONStringArrayEncoding: ParameterEncoding {
    private let myString: String

    init(string: String) {
        self.myString = string
    }

    func encode(_ urlRequest: URLRequestConvertible, with parameters: Parameters?) throws -> URLRequest {
        var urlRequest = urlRequest.urlRequest

        let data = myString.data(using: .utf8)!

        if urlRequest?.value(forHTTPHeaderField: "Content-Type") == nil {
            urlRequest?.setValue("application/json", forHTTPHeaderField: "Content-Type")
        }

        urlRequest?.httpBody = data

        return urlRequest!
    }
}

extension String {
    func deletingPrefix(_ prefix: String) -> String {
        guard self.hasPrefix(prefix) else { return self }
        return String(self.dropFirst(prefix.count))
    }
}
